﻿Imports Infragistics.Win.UltraWinDock

Public Class Form1

#Region "Code to re-sort the flyout tabs"

  Private Sub UltraButton1_Click(sender As System.Object, e As System.EventArgs) Handles UltraButton1.Click

    Me.SuspendLayout()
    UltraDockManager1.SuspendLayout()
    Try
      'Loop over all dock locations of interest, and sort the tabs that are kept in that dock location.
      For Each tmpLocation As DockedLocation In {DockedLocation.DockedLeft, DockedLocation.DockedTop, DockedLocation.DockedRight, DockedLocation.DockedBottom}
        'Note that we sort the tab, and make sure that the tabs that contain the controls that we want at the top of the order are skipped.
        Call SortDockArea(UltraDockManager1, tmpLocation, {Label1, Label2})
      Next
    Finally
      UltraDockManager1.FlyIn()
      UltraDockManager1.ResumeLayout()
      Me.ResumeLayout()
    End Try

  End Sub

  Private Sub SortDockArea(forUltraDockManager As UltraDockManager, forDockedLocation As DockedLocation, noSortControls() As Control)
    'Get the list of DockableControlPanes that ust be held at the top of the sort order based on the controls passed in.
    Dim noSortDockableControlPanes As Generic.List(Of DockableControlPane) = _
      (From tmpControl As Control In noSortControls Select GetPanelForControl(forUltraDockManager, forDockedLocation, tmpControl)).ToList

    'Get a list of DockableControlPanes, in alpha-sort order, that are not members of the 'noSort' list that are docked in a location.
    Dim otherPanels As Generic.List(Of KeyValuePair(Of String, DockableControlPane)) = _
      (From otherPanel As DockableControlPane In forUltraDockManager.ControlPanes _
        Where (Not (otherPanel.Pinned OrElse noSortDockableControlPanes.Contains(otherPanel)) AndAlso (otherPanel.DockAreaPane.DockedLocation = forDockedLocation)) _
        Order By otherPanel.Text Ascending _
        Select New KeyValuePair(Of String, DockableControlPane)(otherPanel.Text.ToUpper, otherPanel) _
      ).ToList()

    'Now that we have our sorted list, we make sure that noSort panels are at the top in reverese order.
    noSortDockableControlPanes.Reverse()
    For Each tmpPane As DockableControlPane In noSortDockableControlPanes
      AddDefinedTabToList(otherPanels, forDockedLocation, tmpPane)
    Next

    'Pin each that's unpinned, and then unpin all those that were unpinned before.
    For Each tmpEntry As KeyValuePair(Of String, DockableControlPane) In otherPanels
      tmpEntry.Value.Tag = tmpEntry.Value.Pinned
      tmpEntry.Value.Pinned = True
    Next
    For Each tmpEntry As KeyValuePair(Of String, DockableControlPane) In otherPanels
      tmpEntry.Value.Pinned = CBool(tmpEntry.Value.Tag)
    Next

  End Sub
  Private Function GetPanelForControl(forUltraDockManager As UltraDockManager, forDockedLocation As DockedLocation, forControl As Control) As DockableControlPane
    Try
      'Get the DockableControlPane that hosts the control.
      Dim tmpDockableControlPane As DockableControlPane = forUltraDockManager.PaneFromControl(forControl)
      'Make sure that the DockableControlPane exists, is unpinned, and is held at the location of interest.
      If ((tmpDockableControlPane IsNot Nothing) AndAlso Not (tmpDockableControlPane.Pinned) AndAlso (tmpDockableControlPane.DockAreaPane.DockedLocation = forDockedLocation)) Then
        Return tmpDockableControlPane
      End If
    Catch ex As Exception
    End Try
    Return Nothing
  End Function
  Private Sub AddDefinedTabToList(panelList As Generic.List(Of KeyValuePair(Of String, DockableControlPane)), forDockedLocation As DockedLocation, panelToAdd As DockableControlPane)
    'If the panel we're adding exists, is not pinned, and is docked in the correct position, we add it.
    If ((panelToAdd IsNot Nothing) AndAlso (panelToAdd.DockAreaPane.DockedLocation = forDockedLocation)) Then
      panelList.Insert(0, New KeyValuePair(Of String, DockableControlPane)(panelToAdd.Text.ToUpper, panelToAdd))
    End If
  End Sub

#End Region 'Code to re-sort the flyout tabs

End Class
